#! /bin/sh
: &&O='cd .' || exec /bin/sh "$0" $argv:q # we're in a csh, feed myself to sh
$O || exec /bin/sh "$0" "$@"              # we're in a buggy zsh

#
# smartlog @version@
# (c) 1994-1997 by Michael 'Bytewurm' Weber (Michael.Weber@rising-systems.de)
#

#{{{script}}}#{{{ init
#{{{ parameters
#{{{ check for -q
verbose="y"
if [ "$1" = "-q" ]
then
  verbose=""
  shift
fi
#}}}
eval "$@"
#}}}
#{{{ variables
sm_get="@installdir@/getmode"
sm_logfile="$sm_user/logfile"
sm_list="$sm_user/files"

LS="@ls@"
RM="@rm@ -f"
MV="@mv@ -f"
CAT="@cat@ --"
SED="@sed@"
AWK="@awk@"
EXPR="@expr@"
GREP="@grep@"
CHOWN="@chown@"
CHGRP="@chgrp@"
CHMOD="@chmod@"
MARKER="`@marker@`"
DIRNAME="@dirname@"

[ "$local" = "no" ] && local=""
[ "$shift" = "no" ] && shift=""
[ "$comp" = "no" ] && comp=""
[ "$ash" = "none" ] && ash=""
[ "$bsh" = "none" ] && bsh=""
[ "$nsh" = "none" ] && nsh=""
[ "$arch" = "none" ] && arch=""

log=""
magiclog=""
suffix=".SLA"
suffix2="$suffix"   # needed for comparisons
identifier="##### SLA #####"

trap 'panic "TRAP-EXIT"' 1 2 3 15
#}}}

#{{{ assign()
assign()
{ fuid="$1"
  fgid="$2"
  fsize="$3"
  fmode="$4"
}
#}}}
#{{{ calc()
calc()
{ if [ -z "$EXPR" ]
  then
    x=$[$*]
  else
    x=`$EXPR $*`
  fi
}
#}}}
#{{{ compress()
compress()
{ if [ "$comp" -a "$ext" ]
  then
    $zip <"$1" >"$1$ext"
    $RM "$1"
    restore "$1$ext"
  else
    restore "$1"
  fi
}
#}}}
#{{{ ftouch()
#
# emulating "touch" using "cat"
# why to use more systools than absolutely necessary?
#
ftouch()
{ $CAT </dev/null >"$1"
}
#}}}
#{{{ move()
#
# shiftlog function
#
move()
{ if [ -f "$1" ]
  then
    $MV "$1" "$2"
    compress "$2"
  elif [ -f "$1$ext" ]
  then
    $MV "$1$ext" "$2$ext"
    restore "$2$ext"
  fi
}
#}}}
#{{{ panic()
panic()
{ echo "$date: Panic: $*." | $SED -e "s/%-%/*/" >>$sm_logfile
  $RM "$tmp" # clear tmpfile
  exit 1
}
#}}}
#{{{ parse_mode()
parse_mode()
{ case $# in
    1) fmode="$1";;
    2|3) [ "x$1" != "x+" ] && fuid="$1"
         [ "x$2" != "x+" ] && fgid="$2"
         [ $# = 3 -a "x$3" != "x+" ] && fmode="$3";;
    *) panic "mode parsing error ... very strange, send bug report";; # probably impossible
  esac
}
#}}}
#{{{ restore()
restore()
{ [ -f "$1" ] || ftouch "$1"
  $CHOWN "$fuid" "$1" 2>/dev/null >/dev/null || warn "chown failed for $fuid ($1)"
  $CHGRP "$fgid" "$1" 2>/dev/null >/dev/null || warn "chgrp failed for $fgid ($1)"
  $CHMOD "$fmode" "$1" 2>/dev/null >/dev/null || warn "chmod failed for $fmode ($1)"
}
#}}}
#{{{ uncompress()
uncompress()
{ if [ -r "$1$ext2" ]
  then
    $zap <"$1$ext2" >"$1"
    $RM "$1$ext2"
    restore "$1"
  fi
}
#}}}
#{{{ warn()
warn()
{ echo "$date: Warning: $*." | $SED -e "s/%-%/*/" >> $sm_logfile
}
#}}}

#{{{ check parameter integrity
#{{{ check/set tmp
if [ -d "$tmp" -a -w "$tmp" ]
then
  tmp="$tmp/smartlog.$$"
else
  panic "$tmp is not a directory or not writable"
fi
$RM "$tmp" # just to be sure we have no tmp-file lazing around
#}}}
#{{{ check zip/zap
if [ "$comp" ] 
then
 [ "x`(echo 'foo' | $zip | $zap) 2>/dev/null`" != "xfoo" ] && panic "zip/zap do not exist or are incompatible"
fi
#}}}
#{{{ check archdir
if [ -z "$local" ]
then
  [ -f "$dir" ] && panic "archive dir $dir is a regular file"
  [ -w "$dir" ] || panic "archive dir $dir does not exist or is unwritable"
fi
#}}}
#{{{ set ext if necessary
#
# "none" is for compatibility to prior versions (2.1 or below)
# "auto" sounds more logical, ain't it?!
#
if [ x"$ext" = x"auto" -o x"$ext" = x"none" ]
then
  #{{{  compress a file and look for the suffix
  #  
  # a file called "99999" should not collide with any other
  # (hopefully :-)
  #  
  echo "--------------------------------------------------------\
        Some programs do not compress empty or very small files.\
        But this size should be ok, even for a dumb 'compress'.
        --------------------------------------------------------" > "99999"
  $zip "99999" >/dev/null 2>&1
  ext=`$LS 99999* | $SED "s/9//g"`
  $RM 99999*
#}}}
  #{{{  if $zip gave no hint, we try something else
  if [ -z "$ext" ]
  then
    case $zip in
      *compress) ext=".Z";;
      *gzip) ext=".gz";;
      *zip) ext=".zip";;
      *freeze) ext=".F";;
      *compact) ext=".C";;  # just for nostalgia :-)
      *) ext=".SLAZ";warn "cannot determine suffix for $zip-$zap, using .SLAZ";;
    esac
  fi
#}}}
fi
ext2="$ext" # needed for comparisons
#}}}
#}}}
#{{{ are we in shifting-mode?
[ "$shift" ] && suffix=""   # yep, no suffix needed
#}}}
#}}}

list=`echo "$logfile" | $SED -e "s/%-%/*/"` # parse wildcards

if [ -z "$list" ]
then
  warn "empty list of files (possible wildcard error)"
else
  for logfile in $list
  do
    #{{{ the work
    #{{{ check logfile
    [ -f "$logfile" ] || panic "$logfile is not a regular file or does not exist"
    [ -r "$logfile" ] || panic "$logfile is not readable"

    if [ -z "$local" ]
    then
      [ -f "$dir" ] && panic "archive dir $dir is not a directory"
      [ -w "$dir" ] || panic "archive dir $dir does not exist or is unwritable"
    fi
#}}}
    #{{{ get the files' mode and size
    #    
    # assign will set fuid, fgid and fmode to the
    # files' permissions which will be used to restore
    # a logfiles' original permissions and ownership
    # fsize wil be set to the files' size
    #    
    assign `$sm_get $logfile`

    if [ "x$mode" != "x+.+/+" ]
    then
      OIFS="$IFS"
      IFS="./"
      parse_mode $mode
      IFS="$OIFS"
    fi
#}}}
    #{{{ archive to use
    if [ -z "$arch" ]
    then
      #{{{ no alternative archive given
      if [ "$local" ]
      then
        archive="$logfile"
      else
        archive="$dir/"`echo "$logfile" | $SED -e "s/\//./g" -e "/./s/.//"`
      fi
      archive2="$archive" # save archive without suffix
      archive="$archive$suffix"
#}}}
    else
      #{{{ alternative archive given
      if [ "$local" ]
      then
        archive=`$DIRNAME "$logfile"`"/$arch"
      else
        archive="$dir/$arch"
      fi
      archive2="$archive" # save archive without suffix
#}}}
    fi
#}}}
    #{{{ write stuff to filelist
    echo "$logfile:$archive:$shift:$zap:$ext" >>"$sm_list"
#}}}

    if [ "$fsize" -ge "$size" ]
    then
      #{{{ preparatory work
      #{{{ checkups
      #{{{ archivepath
      [ ! -w `$DIRNAME $archive` ] && panic "invalid archive-path ($archive)"
#}}}
      #{{{ logfile==archive
      [ x"$archive" = x"$logfile" -a ! "$shift" ] && panic "archive and logfile are identical for $logfile"
#}}}
      #{{{ double archives
      #      
      # these checks are *very* rough
      # but should suffice
      #      
      [ -f "$archive" -a -f "$archive2$suffix2" -a \
        x"$archive" != x"$archive2$suffix2" -a \
        x"$archive" != x"$logfile" \
      ] && panic "double archive $archive & $archive2$suffix2"

      [ -f "$archive" -a -f "$archive2" -a \
        x"$archive" != x"$archive2" -a \
        x"$archive2" != x"$logfile" \
      ] && panic "double archive $archive & $archive2"
#}}}
#}}}
      #{{{ a present for archiving junkies
      #      
      # if someone archives smartlog-archives (no comment :-)
      # we have to adapt the identifier and the delimiter
      # this will even work for the archive of archives of archives ...
      #      
      while [ `$GREP -c "$identifier" "$logfile"` -gt 0 ]
      do
        identifier="#####$identifier"    # grow identifier significantly
      done

      delimiter="$identifier$MARKER========== $date =========="
#}}}
#}}}
      #{{{ archiving routines
      #{{{ bsh
      if [ "$bsh" ]
      then
        (eval $bsh 2>>"$sm_logfile") || warn "bsh: '$bsh' exited with status $?"
      fi
#}}}
      if [ "$shift" ]
      then
        #{{{ shifting mode
        #        
        # original shiftlog routines by Stephen R. van den Berg
        # (modified)
        #        
        #{{{ convert standard-archive if any
        #{{{ determine archive to convert
        if [ "$arch" ]
        then
          convarch="$archive"
        else
          convarch="$archive$suffix2"
        fi
        uncompress "$convarch"
#}}}
        if [ -f "$convarch" ]
        then
          #{{{ panic on existing shift-archives
          #          
          # this will kick you out if you somehow made it to create
          # standard and shift archives with the same name
          #          
          [ -f "$archive2.1" ] && panic "convert: $archive2.* already exist"
#}}}
          #{{{ awk does it
          #          
          # just in case archives have been manipulated we look
          # for the first identifier explicitely
          #          
          $AWK -v IDENT="$identifier" -v DEST="$archive2"\
            'BEGIN { i=0; ok=0 }\
            index($0,IDENT)==1 { close(DEST"."i); i=i+1; ok=1; next }\
            ok==1 { print >>DEST"."i }' <"$convarch"
#}}}
          $RM "$convarch"
          #{{{ give all archives the right mode
          x=1
          while [ -f $archive.$x ]
          do
            restore "$archive.$x"
            calc $x + 1
          done
#}}}
          magiclog=$magiclog"/$convarch split into $x files"
        fi
#}}}
        #{{{ compress || !compress
        if [ "$comp" ]
        then
          case "$comp" in
            *[!0-9]*) firstcomp=0;;  # no integer, compress all archives
            *) firstcomp=$comp;; # interpret integer variables
          esac
        else
          ext=""
          firstcomp=0 # make sure integer comparisons won't fail
        fi
#}}}
        #{{{ cycle old archives
        j=1  # this is for a standard cycle

        #        
        # we look for 0-archives which have been created by versions <3.2
        # and adapt the stuff from 0-n to 1-(n+1) and cycle archives
        # simultaneously from 1-m to 2-(m+1)
        #        
        # doing this in a single loop is much more efficient than using a
        # special procedure
        #        
        [ -f "$archive.0" -o -f "$archive.0$ext2" ] && j=2

        i="$hold"  # start with the HIGHEST value (=LAST archive)
        while [ $i -ge 2 ]
        do
          calc $i - $j
          #{{{  uncompress archive if necessary
          #          
          # archives with minor numbers maybe compressed
          # and we uncompress them now if necessary
          #          
          [ -z "$ext" ] && uncompress "$archive.$x"
#}}}
          move "$archive.$x" "$archive.$i"
          [ $i = $firstcomp ] && ext=""  # no compression on minor numbers
          calc $i - 1
          i=$x
        done
#}}}
        #{{{ move the logfile to the archive
        move "$logfile" "$archive.1"
        restore "$logfile"    # recreate $logfile with its exact permissions
#}}}
        #{{{ kill overhanging shift-archives
        calc $hold + 1  # we look for archives >max hold

        while [ -f "$archive.$x" -o -f "$archive.$x$ext2" ]
        do
        #          
        # I know its not nice to remove both if one was detected,
        # but it makes the stuff more readable
        #          
          $RM "$archive.$x" "$archive.$x$ext2"
          calc $x + 1
        done
#}}}
#}}}
      else
        #{{{ standard mode
        #{{{ uncompress archive if any
        #        
        # heaven, don't ask me why I coded this :-)
        # it makes sure EVERY archive we process is uncompressed
        # whether it is userdefined or not
        # does not seem to be necessary, but better doing too much :-))
        #         
        uncompress "$archive"
        uncompress "$archive2$suffix"
#}}}
        #{{{ panic on existing standard AND shift archive
        #        
        # this will kick you out if you somehow made it to create
        # standard and shift archives with the same name
        #        
        [ -f "$archive" -a -f "$archive2.1" ] && panic "convert: $archive already exists"
#}}}
        #{{{ convert shift-archives if any
        x=1
        $RM "$tmp" # just to be sure
        while [ -f "$archive2.$x" -o -f "$archive2.$x$ext2" ]
        do
          #{{{ collect archives in tmp-file
          uncompress "$archive2.$x"
          echo "$delimiter" >> "$tmp"
          $CAT "$archive2.$x" >> "$tmp"
          $RM "$archive2.$x"
          calc $x + 1
#}}}
        done
        #{{{ save stuff if we have some
        if [ $x -gt 1 ]
        then
          $MV "$tmp" "$archive"
          magiclog=$magiclog"/$x files collected"
        fi
#}}}
#}}}
        #{{{ restore archives' permissions or touch it
        restore "$archive"
#}}}
        #{{{ delete old entries
        held=`$GREP -c "$identifier" "$archive" 2>/dev/null`
        calc $held - $hold + 1

        if [ $x -gt 0 ]
        then
          #{{{ awk call to cut the leading generations
          $AWK -v IDENT="$identifier" -v DEL="$x"\
           'BEGIN { DEL=DEL+1 }\
            DEL==0 { print; next }\
            index($0,IDENT)==1 { DEL=DEL-1 }\
            DEL==0 { print }' <"$archive" >"$tmp"
#}}}
          $MV "$tmp" "$archive" # move modified file back to origin
        fi
#}}}
        #{{{ collect archive and logfile in a new archive
        echo "$delimiter" >> "$archive"
        $CAT "$archive" "$logfile" > "$tmp"
        $CAT </dev/null >"$logfile"  # this saves the logfiles' permissions
        $MV "$tmp" "$archive"
        restore "$archive"   # we want archives with logfile-permissions
        compress "$archive"  # compress archive if necessary
#}}}
#}}}
      fi
      #{{{ ash
      if [ "$ash" ]
      then
        (eval $ash 2>>"$sm_logfile") || warn "ash: '$ash' exited with status $?"
      fi
#}}}
#}}}
      #{{{ diagnostic output for verbose mode
      if [ "$verbose" ]
      then
        #{{{ rig up output
        log="$logfile ($fsize bytes) --> $archive (maxhold $hold)"
        [ "$local" ] && log=$log"/local"
        [ "$shift" ] && log=$log"/shift"
        if [ "$comp" ]
        then
         log=$log"/compress"
         [ "$shift" ] && log=$log" (first #$firstcomp)"
        fi
        [ "$bsh" ] && log=$log"/bsh='$bsh'"
        [ "$ash" ] && log=$log"/ash='$ash'"
        [ "$nsh" ] && log=$log"/(nsh='$nsh')"
        log="$log$magiclog" # append info from automagical functions
#}}}
        echo "$date: $log" >>"$sm_logfile"
      fi
#}}}
    else
      #{{{ file too small, execute nsh?
      if [ "$verbose" ]
      then
        if [ "$fsize" = 0 ]
        then
          echo "$date: $logfile is empty." >>$sm_logfile
        else
          echo "$date: $logfile has size below $size bytes." >>$sm_logfile
        fi
      fi

      if [ "$nsh" ]
      then
        [ "$verbose" ] && echo "$date: executing '$nsh' instead." >>$sm_logfile
        (eval $nsh 2>>"$sm_logfile") || warn "nsh: '$nsh' exited with status $?"
      fi
#}}}
    fi

    $RM "$tmp" # just to be *really* sure
#}}}
  done
fi

exit 0
