#!/bin/sh
: &&O='cd .' || exec /bin/sh "$0" $argv:q # we're in a csh, feed myself to sh
$O || exec /bin/sh "$0" "$@"              # we're in a buggy zsh

#
# smartlog  (c) 1994 by Michael Weber
#

#{{{}}}
#{{{  init
#{{{  make sure we get decent shell
if [ -z "$IFS" ]
then IFS=" \
	\

"
  export IFS
fi

SHELL=/bin/sh
export SHELL
#}}}

#{{{  panic()
panic()
{ 1>&2
  echo "Panic: $*. Aborted."
  exit 1
}
#}}}
#{{{  whitch()
#
# avoiding strange which-formats, we import our own.
# it's magic :-)
#
whitch()
{ OIFS="$IFS"
  IFS=":"
  found=""
  for xx in $PATH
  do
    if [ -x "$xx/$1" ]
    then
      found="$xx/$1"
      break
    fi
  done
  IFS="$OIFS"
}
#}}}
#{{{  makedir()
#
# some systems do not support mkdir -p
# or it does not work properly, so we import our own
# must be called in a subshell!
#
makedir()
{ cd /
  IFS="/"
  for xx in $1
  do
    if [ "$xx" ]
    then
      [ -d $xx ] || (mkdir "$xx" || panic "dir-creation failed for $xx of $1")
      cd $xx || panic "$xx of $1 not accessable"
    fi
  done
}
#}}}
#{{{  checkawk()
checkawk()
{ [ "x`$awk -v foo=2 '\
    BEGIN {\
      if (match(\"-foo\",\"foo\")==2 && system(\"exit 2\")==2)\
       print foo;\
    } \
    { print } \
    ' </dev/null`" != "x2" ] && awk=""
}
#}}}

#{{{  version
version="V2.3"
#}}}
#{{{  check test
#
# these are the 'test' features we need
# I don't think the test is really necessary, but better doing too much
#

tmp="./foo.$$"
cat </dev/null >$tmp
chmod 755 $tmp

for opt in f x r w
do
  [ -$opt $tmp ] || panic "'test' is invalid for option '-$opt'"
done

[ -s $tmp ] && panic "'test' is invalid for option '-s'"
[ -d . ] || panic "'test' is invalid for '-d'"
[ 1 = 1 -a xy != x ] || panic "'test' is invalid for comparisons"
[ 2 -ge 1 -a 2 -ge 2 ] || panic "'test' is invalid for option '-ge'"
[ 2 -gt 1 ] || panic "'test' is invalid for option '-gt'"

rm -f $tmp
#}}}
#}}}
#{{{  get install path
clear
echo ""
echo "smartlog $version"
echo "-install.sh in progress-"
echo ""
echo ""
echo "Which directory to use for installation? [CR=/usr/local/etc/smartlog] :"
read sm_path

[ ! "$sm_path" ] && sm_path="/usr/local/etc/smartlog"

dpath=`dirname $sm_path`
while [ ! -w $dpath -a ! -z $dpath -a $dpath != "/" -a $dpath != "." ]
do
  dpath=`dirname $dpath`
done

[ -f $sm_path -o $dpath = "." -o ! -w $dpath ] && \
   panic "invalid directory $sm_path"
#}}}
#{{{  get man path
echo""
echo "Where to install the manpage? [CR=/usr/local/man/man1] :"
read man_path
[ ! "$man_path" ] && man_path="/usr/local/man/man1"

mpath=`dirname $man_path`
while [ ! -w $mpath -a ! -z $mpath -a $mpath != "/" -a $mpath != "." ]
do
  mpath=`dirname $mpath`
done

[ -f "$man_path" -o $mpath = "."  -o "$sm_path" != "$man_path" -a ! -w "$mpath" ] \
 && panic "invalid man-dir $man_path"
#}}}
#{{{  check system tools
echo""
echo "Checking system tools..."
#{{{  standard tools
for systool in ls rm mv cat sed grep mkdir chmod chown chgrp dirname
do
  whitch "$systool"
#  {{{  found || ! found ?
  if [ -z "$found" ]
  then
    panic "'$systool' is missing in the path"
  else
    echo "$systool is $found"
    eval "$systool=$found"
  fi
#  }}}
done
#}}}
#{{{  fgrep
#
# grep should be installed everywhere, but what about fgrep?
#
whitch "fgrep"
[ -z "$found" ] || grep="$found"
#}}}
#{{{  fgres
fgres=""
whitch "fgres"
[ -z "$found" ] ||\
  $found -i -F "" -r /dev/null -w /dev/null "" 2>/dev/null && fgres="$found"
#}}}
#{{{  $[]-builtin
echo ""
([ $[1 + 1] = 2 ] 2>/dev/null)

if [ $? != 0 ]
then
#  {{{  $[] not available, checkout 'expr'
  whitch "expr"
  if [ -z "$found" ]
  then
    panic "'expr' is missing in the path"
  else
    expr="$found"
    echo "using $expr for calculations"
  fi
#  }}}
else
  expr=""
  echo "using \$[]-builtin for calculations"
fi
#}}}
#}}}
#{{{  look for awk
echo""
echo "Looking for an awk to use..."
awk=""
for i in awk gawk mawk
do
   echo "Checking $i."
   whitch $i
   $found '{ print }' </dev/null >/dev/null 2>/dev/null
#   {{{  a valid choice?
   if [ $? = 0 ]
   then
     awk="$found"
     checkawk
   fi
#   }}}
done
#{{{  ask for awk if none found
if [ ! "$awk" ]
then
  echo ""
  echo "I could not find a posix conform awk. Which awk should I use? :"
  read awk
  $awk '{ print }' </dev/null >/dev/null 2>/dev/null || panic "$awk not found"
  checkawk
  [ ! "$awk" ] && panic "Sorry, this awk is not posix conform"
fi

echo "$awk seems to be posix conform. I'll use it."
#}}}
#}}}
#{{{  install files in $sm_path
echo ""
echo "Installing smartlog in $sm_path,"

[ -d $sm_path ] || (makedir $sm_path)

rm -f $sm_path/getmode
cp getmode $sm_path
#}}}
#{{{  substitutions
if [ "$fgres" ]
then
#  {{{  use fgres for substitutions
  echo "using 'fgres' for substitutions."
  for file in smartlog* rc.*
  do
#    {{{  the fgres-substitutions
    fgres -e ">>>sm_path<<< $sm_path" \
          -e ">>>ls<<< $ls" \
          -e ">>>rm<<< $rm" \
          -e ">>>mv<<< $mv" \
          -e ">>>awk<<< $awk" \
          -e ">>>cat<<< $cat" \
          -e ">>>sed<<< $sed" \
          -e ">>>expr<<< $expr" \
          -e ">>>grep<<< $grep" \
          -e ">>>mkdir<<< $mkdir" \
          -e ">>>chmod<<< $chmod" \
          -e ">>>chgrp<<< $chgrp" \
          -e ">>>chown<<< $chown" \
          -e ">>>dirname<<< $dirname" \
          -e ">>>version<<< $version" \
    <$file >$sm_path/$file
#    }}}
  done
#  }}}
else
#  {{{  use sed for substitutions
  echo "using 'sed' for substitutions."
#  {{{  prepare quoted vars
#  
#   we have to quote /'s to avoid sed-confusion
#   hard job ain't it? :-)
#  
  for conv in sm_path awk ls rm mv cat sed expr grep mkdir chmod chown chgrp dirname
  do
    eval "tmp=$"$conv""
    eval "q$conv=`echo $tmp | sed 's/\//\\\\\\\\\//g'`"
  done
#  }}}
  for file in smartlog* rc.*
  do
#    {{{  the sed-substitutions
    sed -e "s/>>>sm_path<<</$qsm_path/g" \
        -e "s/>>>ls<<</$qls/" \
        -e "s/>>>rm<<</$qrm/" \
        -e "s/>>>mv<<</$qmv/" \
        -e "s/>>>cat<<</$qcat/" \
        -e "s/>>>sed<<</$qsed/" \
        -e "s/>>>awk<<</$qawk/" \
        -e "s/>>>expr<<</$qexpr/" \
        -e "s/>>>grep<<</$qgrep/" \
        -e "s/>>>mkdir<<</$qmkdir/" \
        -e "s/>>>chmod<<</$qchmod/" \
        -e "s/>>>chgrp<<</$qchgrp/" \
        -e "s/>>>chown<<</$qchown/" \
        -e "s/>>>dirname<<</$qdirname/" \
        -e "s/>>>version<<</$version/" \
    <$file >$sm_path/$file
#    }}}
  done
#  }}}
fi
#}}}
#{{{  finish installation
chmod 755 $sm_path/smartlog $sm_path/smartlog.sh $sm_path/getmode

chmod 644 $sm_path/smartlog.awk $sm_path/smartlog.1 \
          $sm_path/smartlog.def $sm_path/rc.*

if [ "$man_path" != "$sm_path" ]
then
  (makedir $man_path)
  mv -f "$sm_path/smartlog.1" "$man_path"
fi

echo ""
echo "Done."
#}}}
